@@ -3,7 +3,7 @@ import logging
 
 from .about import __version__
 from .candidate import Candidate
-from .ice import Connection, ConnectionClosed
+from .ice import Connection, ConnectionClosed, TransportPolicy
 
 # Set default logging handler to avoid "No handler found" warnings.
 logging.getLogger(__name__).addHandler(logging.NullHandler())
