@@ -30,6 +30,20 @@ protocol_id = count()
 _mdns = threading.local()
 
 
+class TransportPolicy(enum.Enum):
+    ALL = 0
+    """
+    All ICE candidates will be considered.
+    """
+
+    RELAY = 1
+    """
+    Only ICE candidates whose IP addresses are being relayed,
+    such as those being passed through a STUN or TURN server,
+    will be considered.
+    """
+
+
 async def get_or_create_mdns_protocol(subscriber: object) -> mdns.MDnsProtocol:
     if not hasattr(_mdns, "lock"):
         _mdns.lock = asyncio.Lock()
@@ -282,6 +296,7 @@ class Connection:
     :param turn_transport: The transport for TURN server, `"udp"` or `"tcp"`.
     :param use_ipv4: Whether to use IPv4 candidates.
     :param use_ipv6: Whether to use IPv6 candidates.
+    :param transport_policy: Transport policy.
     """
 
     def __init__(
@@ -296,6 +311,7 @@ class Connection:
         turn_transport: str = "udp",
         use_ipv4: bool = True,
         use_ipv6: bool = True,
+        transport_policy: TransportPolicy = TransportPolicy.ALL,
     ) -> None:
         self.ice_controlling = ice_controlling
         #: Local username, automatically set to a random value.
@@ -325,6 +341,7 @@ class Connection:
         self._early_checks: List[
             Tuple[stun.Message, Tuple[str, int], StunProtocol]
         ] = []
+        self._early_checks_done = False
         self._event_waiter: Optional[asyncio.Future[ConnectionEvent]] = None
         self._id = next(connection_id)
         self._local_candidates: List[Candidate] = []
@@ -341,6 +358,17 @@ class Connection:
         self._use_ipv4 = use_ipv4
         self._use_ipv6 = use_ipv6
 
+        if (
+            stun_server is None
+            and turn_server is None
+            and transport_policy == TransportPolicy.RELAY
+        ):
+            raise ValueError(
+                "Relay transport policy requires a STUN and/or TURN server."
+            )
+
+        self._transport_policy = transport_policy
+
     @property
     def local_candidates(self) -> List[Candidate]:
         """
@@ -469,6 +497,7 @@ class Connection:
         for early_check in self._early_checks:
             self.check_incoming(*early_check)
         self._early_checks = []
+        self._early_checks_done = True
 
         # perform checks
         while True:
@@ -854,13 +883,17 @@ class Connection:
         for address in addresses:
             # create transport
             try:
-                _, protocol = await loop.create_datagram_endpoint(
+                transport, protocol = await loop.create_datagram_endpoint(
                     lambda: StunProtocol(self), local_addr=(address, 0)
                 )
+                sock = transport.get_extra_info("socket")
+                if sock is not None:
+                    sock.setsockopt(
+                        socket.SOL_SOCKET, socket.SO_RCVBUF, turn.UDP_SOCKET_BUFFER_SIZE
+                    )
             except OSError as exc:
                 self.__log_info("Could not bind to %s - %s", address, exc)
                 continue
-            protocol = cast(StunProtocol, protocol)
             host_protocols.append(protocol)
 
             # add host candidate
@@ -874,7 +907,8 @@ class Connection:
                 port=candidate_address[1],
                 type="host",
             )
-            candidates.append(protocol.local_candidate)
+            if self._transport_policy == TransportPolicy.ALL:
+                candidates.append(protocol.local_candidate)
         self._protocols += host_protocols
 
         # query STUN server for server-reflexive candidates (IPv4 only)
@@ -906,7 +940,6 @@ class Connection:
                 ssl=self.turn_ssl,
                 transport=self.turn_transport,
             )
-            protocol = cast(StunProtocol, protocol)
             self._protocols.append(protocol)
 
             # add relayed candidate
@@ -1018,7 +1051,7 @@ class Connection:
         response.add_message_integrity(self.local_password.encode("utf8"))
         protocol.send_stun(response, addr)
 
-        if not self._check_list:
+        if not self._check_list and not self._early_checks_done:
             self._early_checks.append((message, addr, protocol))
         else:
             self.check_incoming(message, addr, protocol)
