@@ -1,9 +1,21 @@
 import asyncio
 import hashlib
 import logging
+import socket
 import struct
 import time
-from typing import Any, Callable, Dict, List, Optional, Text, Tuple, Union, cast
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Text,
+    Tuple,
+    TypeVar,
+    Union,
+    cast,
+)
 
 from . import stun
 from .utils import random_transaction_id
@@ -14,6 +26,9 @@ DEFAULT_CHANNEL_REFRESH_TIME = 500
 DEFAULT_ALLOCATION_LIFETIME = 600
 TCP_TRANSPORT = 0x06000000
 UDP_TRANSPORT = 0x11000000
+UDP_SOCKET_BUFFER_SIZE = 262144
+
+_ProtocolT = TypeVar("_ProtocolT", bound=asyncio.BaseProtocol)
 
 
 def is_channel_data(data: bytes) -> bool:
@@ -35,6 +50,7 @@ class TurnStreamMixin:
 
         while len(self.buffer) >= 4:
             _, length = struct.unpack("!HH", self.buffer[0:4])
+            length += stun.padding_length(length)
             if is_channel_data(self.buffer):
                 full_length = 4 + length
             else:
@@ -46,6 +62,13 @@ class TurnStreamMixin:
             self.datagram_received(self.buffer[0:full_length], addr)
             self.buffer = self.buffer[full_length:]
 
+    def _padded(self, data: bytes) -> bytes:
+        # TCP and TCP-over-TLS must pad messages to 4-byte boundaries.
+        padding = stun.padding_length(len(data))
+        if padding:
+            data += bytes(padding)
+        return data
+
 
 class TurnClientMixin:
     _send: Callable
@@ -304,7 +327,7 @@ class TurnClientTcpProtocol(TurnClientMixin, TurnStreamMixin, asyncio.Protocol):
     """
 
     def _send(self, data: bytes) -> None:
-        self.transport.write(data)
+        self.transport.write(self._padded(data))
 
     def __repr__(self) -> str:
         return "turn/tcp"
@@ -369,7 +392,7 @@ class TurnTransport:
 
 
 async def create_turn_endpoint(
-    protocol_factory: Callable,
+    protocol_factory: Callable[[], _ProtocolT],
     server_addr: Tuple[str, int],
     username: Optional[str],
     password: Optional[str],
@@ -377,11 +400,13 @@ async def create_turn_endpoint(
     channel_refresh_time: int = DEFAULT_CHANNEL_REFRESH_TIME,
     ssl: bool = False,
     transport: str = "udp",
-) -> Tuple[TurnTransport, asyncio.Protocol]:
+) -> Tuple[TurnTransport, _ProtocolT]:
     """
     Create datagram connection relayed over TURN.
     """
     loop = asyncio.get_event_loop()
+    inner_protocol: asyncio.BaseProtocol
+    inner_transport: asyncio.BaseTransport
     if transport == "tcp":
         inner_transport, inner_protocol = await loop.create_connection(
             lambda: TurnClientTcpProtocol(
@@ -406,6 +431,9 @@ async def create_turn_endpoint(
             ),
             remote_addr=server_addr,
         )
+        sock = inner_transport.get_extra_info("socket")
+        if sock is not None:
+            sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, UDP_SOCKET_BUFFER_SIZE)
 
     try:
         protocol = protocol_factory()
