@@ -5,7 +5,7 @@ import socket
 import unittest
 from unittest import mock
 
-from aioice import Candidate, ice, mdns, stun
+from aioice import Candidate, TransportPolicy, ice, mdns, stun
 
 from .turnserver import run_turn_server
 from .utils import asynctest, invite_accept
@@ -267,6 +267,51 @@ class IceConnectionTest(unittest.TestCase):
         await conn_a.close()
         await conn_b.close()
 
+    @asynctest
+    async def test_connect_early_checks_2(self):
+        conn_a = ice.Connection(ice_controlling=True)
+        conn_b = ice.Connection(ice_controlling=False)
+
+        # both sides gather local candidates and exchange credentials
+        await conn_a.gather_candidates()
+        await conn_b.gather_candidates()
+        conn_a.remote_username = conn_b.local_username
+        conn_a.remote_password = conn_b.local_password
+        conn_b.remote_username = conn_a.local_username
+        conn_b.remote_password = conn_a.local_password
+
+        async def connect_b():
+            # side B receives offer and connects
+            for candidate in conn_a.local_candidates:
+                await conn_b.add_remote_candidate(candidate)
+            await conn_b.add_remote_candidate(None)
+            await conn_b.connect()
+
+            # side A receives candidates
+            for candidate in conn_b.local_candidates:
+                await conn_a.add_remote_candidate(candidate)
+            await conn_a.add_remote_candidate(None)
+
+        # The sequence is:
+        # - side A starts connecting immediately, but has no candidates
+        # - side B receives candidates and connects
+        # - side A receives candidates, and connection completes
+        await asyncio.gather(conn_a.connect(), connect_b())
+
+        # send data a -> b
+        await conn_a.send(b"howdee")
+        data = await conn_b.recv()
+        self.assertEqual(data, b"howdee")
+
+        # send data b -> a
+        await conn_b.send(b"gotcha")
+        data = await conn_a.recv()
+        self.assertEqual(data, b"gotcha")
+
+        # close
+        await conn_a.close()
+        await conn_b.close()
+
     @asynctest
     async def test_connect_two_components(self):
         conn_a = ice.Connection(ice_controlling=True, components=2)
@@ -1155,6 +1200,49 @@ class IceConnectionTest(unittest.TestCase):
         await conn.gather_candidates()
         self.assertEqual(conn.local_candidates, [])
 
+    @asynctest
+    async def test_gather_candidates_relay_only_no_servers(self):
+        with self.assertRaises(ValueError) as cm:
+            ice.Connection(ice_controlling=True, transport_policy=TransportPolicy.RELAY)
+        self.assertEqual(
+            str(cm.exception),
+            "Relay transport policy requires a STUN and/or TURN server.",
+        )
+
+    @asynctest
+    async def test_gather_candidates_relay_only_with_stun_server(self):
+        async with run_turn_server() as stun_server:
+            conn_a = ice.Connection(
+                ice_controlling=True,
+                stun_server=stun_server.udp_address,
+                transport_policy=TransportPolicy.RELAY,
+            )
+            conn_b = ice.Connection(ice_controlling=False)
+
+            # invite / accept
+            await invite_accept(conn_a, conn_b)
+
+            # we whould only have a server-reflexive candidate in connection a
+            self.assertCandidateTypes(conn_a, set(["srflx"]))
+
+    @asynctest
+    async def test_gather_candidates_relay_only_with_turn_server(self):
+        async with run_turn_server(users={"foo": "bar"}) as turn_server:
+            conn_a = ice.Connection(
+                ice_controlling=True,
+                turn_server=turn_server.udp_address,
+                turn_username="foo",
+                turn_password="bar",
+                transport_policy=TransportPolicy.RELAY,
+            )
+            conn_b = ice.Connection(ice_controlling=False)
+
+            # invite / accept
+            await invite_accept(conn_a, conn_b)
+
+            # we whould only have a server-reflexive candidate in connection a
+            self.assertCandidateTypes(conn_a, set(["relay"]))
+
     @asynctest
     async def test_repr(self):
         conn = ice.Connection(ice_controlling=True)
